#include "project.h"
#if !i2c__DISABLED
#include "stdio.h"
#include "stdlib.h"
#include "tty_utils.h"
#include "i2c_utils.h"

static int      i2c_keep_on  = 1 ;
static uint16_t i2c_loop     = 1 ;
static uint16_t i2c_interval = 100 ; /* 100 ms */
static uint8_t  i2c_slave_address = 0x08 ; 
static int      i2c_delay    = 30 ; /* default 30ms */
uint16_t        timeoutms = 500 ;


f_list_type i2c_func_list[] = {
    { "help",      do_i2c_help,       "show help message" },
    { "loop",      do_i2c_loop,       "set loop counts"},
    { "interval",  do_i2c_interval,   "set interval time (ms)"},
    { "scan",      do_i2c_scan,       "scan I2C bus"      },
    { "slave",     do_i2c_slave,      "set slave address" },
    { "read",      do_i2c_read_byte,  "read 1 byte from the slave" },
    { "write",     do_i2c_write_byte, "write 1 byte to the slave" },
    { "writereg",  do_i2c_write_reg,  "write 1 byte to reg" },
    { "readreg",   do_i2c_read_reg,   "read 1 byte from reg" },
    { "readregs",  do_i2c_read_regs,  "read n bytes starting from the reg" },
    { "delay",     do_i2c_delay,      "set delay between address and command" },
    { "status",    do_i2c_status,     "print config status" },
    { "return",    do_i2c_return,     "return to the previous mode" },
    { 0,          0,             0} /* list terminator */
} ;

void do_i2c(void)
{
    func_ptr func ;
    i2c_keep_on = 1 ;
    
    I2C_Start() ;
    
    do_i2c_help() ;
    
    prompt("i2c> ") ;
    
    while(i2c_keep_on) {
        if (get_line()) {
            func = get_func(i2c_func_list, str) ;
            if (func) {
                func() ;
            } else {
                print("Unknown Command: ") ;
                print(str) ;
                print("\n\r") ;
                do_i2c_help() ;
            }
            if (i2c_keep_on) {
                prompt("i2c> ") ;
            }
        }
    }
}


void do_i2c_help(void) 
{
    show_help(i2c_func_list) ;    
}

void do_i2c_return(void) 
{
    i2c_keep_on = 0 ;
}

void do_i2c_loop(void) 
{
    sscanf(str, "%*s %hd", &i2c_loop) ;
}

void do_i2c_interval(void) 
{
    sscanf(str, "%*s %hd", &i2c_interval) ;
}

void do_i2c_delay(void)
{
    sscanf(str, "%*s %d", &i2c_delay) ;
}

void do_i2c_status(void) 
{
    snprintf(str, STR_BUF_LEN, "I2C Config: slave address: 0x%02X (%d)\n\r",
        i2c_slave_address, i2c_slave_address) ;
    print(str) ;
}

void do_i2c_scan(void) 
{
    uint8_t address ;
    print("Scanning I2C Addresses... \n\r") ;
    for (address = 0 ; address < 127 ; address++ ) {
        if (i2c_test_address(address)) {
            snprintf(str, STR_BUF_LEN, "0x%02X : device present\r\n", address) ;
            print(str) ;
        }
    }
    print("Scan completed\n\r") ;
}

void do_i2c_slave(void) 
{
    int slave_address ;
    sscanf(str, "%*s %x", &slave_address) ;
    i2c_set_slave_address(slave_address) ;
}

void do_i2c_read_byte(void) 
{
    int i ;
    uint8_t value ;
    uint8_t status ;
    
    for (i = 0 ; i < i2c_loop ; i++ ) {
        status = i2c_read_byte(&value) ;
        if (status == I2C_MSTR_NO_ERROR) {
            snprintf(str, STR_BUF_LEN, "0x%02X\n\r", value) ;
        } else {
            snprintf(str, STR_BUF_LEN, "I2C Read Byte Failed\n\r") ;
        }
        CyDelay(i2c_interval) ;
    }
    print(str) ;
}

void do_i2c_write_byte(void) 
{
    uint8_t status ;
    int value ;
    sscanf(str, "%*s %x", &value) ;
    status = i2c_write_byte(value & 0xFF) ;
    if (status == I2C_MSTR_NO_ERROR) {
        print("OK\n\r") ;
    } else {
        snprintf(str, STR_BUF_LEN, "I2C Write Byte Failed: 0x%X\n\r", status) ;
    }
}

void do_i2c_read_reg(void) 
{
    int i ;
    int reg_address ;
    uint8_t value ;
    uint8_t status ;
    
    sscanf(str, "%*s %x", &reg_address) ;
    
    for (i = 0 ; i < i2c_loop ; i++ ) {
        CyDelay(i2c_interval) ;
        status = i2c_read_reg(reg_address, &value) ;
        if (status == I2C_MSTR_NO_ERROR) {
            snprintf(str, STR_BUF_LEN, "0x%02X (%d) \n\r",
                value, value) ;
        } else {
            snprintf(str, STR_BUF_LEN, "I2C Read Reg at 0x%02X failed\n\r",
                reg_address) ;
        }
    }
    print(str) ;
}

void do_i2c_write_reg(void) 
{
    int reg_address ;
    int value ;
    sscanf(str, "%*s %x %x", &reg_address, &value) ;
    i2c_write_reg(reg_address, value) ;
}

void do_i2c_read_regs(void) 
{
    int loop ;
    int reg_address ;
    int num, i ;
    uint8_t *data ;
    sscanf(str, "%*s %x %d", &reg_address, &num) ;
    data = (uint8_t *)malloc(sizeof(uint8_t) * num) ;
    if (data == 0) {
        print("Not enough memory\r\n") ;
        return ;
    }
    for (loop = 0 ; loop < i2c_loop ; loop++ ) {
        CyDelay(i2c_interval) ;
        i2c_read_regs(reg_address, data, num) ;
        for (i = 0 ; i < num ; i++) {
            snprintf(str, STR_BUF_LEN, "%02X ", data[i] & 0xFF) ;
            print(str) ;
        }
        print("\n\r") ;
    }
    free( data ) ;
}

void i2c_set_slave_address(uint8_t slave_address)
{
    i2c_slave_address = slave_address ;
}

int i2c_test_address(uint8_t address) 
{
    int result = 0 ;
    uint8_t errStatus = I2C_MSTR_NO_ERROR ;
    
    errStatus = I2C_MasterSendStart(address, 1) ; /* read access */   
    if (errStatus == I2C_MSTR_NO_ERROR) {
        result = 1 ;
    }
    errStatus = I2C_MasterSendStop() ;    
    return(result) ;
}

int i2c_read_byte(uint8_t *value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 1) ; /* read access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        *value = I2C_MasterReadByte(I2C_ACK_DATA) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Read Byte Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;
    return(errStatus) ;
}

int i2c_write_byte(uint8_t value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(value) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Write Byte Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;
    return(errStatus) ;    
}

int i2c_read_reg(uint8_t reg_address, uint8_t *value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterSendRestart(i2c_slave_address, 1) ; /* read access */
    } else {
        print("I2C Write Reg Address Failed\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        *value = I2C_MasterReadByte(I2C_NAK_DATA) ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Read Reg Data Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  
    return( errStatus ) ;
}

int i2c_write_reg(uint8_t reg_address, uint8_t value) 
{
    uint8 errStatus = I2C_MSTR_NO_ERROR ;
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(value) ;
    }
    if (errStatus != I2C_MSTR_NO_ERROR) {
        print("I2C Write Reg Data Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  
    return( errStatus ) ;
}

int i2c_read_regs(uint8_t reg_address, uint8_t *value, int num) 
{
    int i ;
    uint8 errStatus = I2C_MSTR_NO_ERROR ;

#if 0
    for (i = 0 ; i < num ; i++ ) {
        errStatus = i2c_read_reg(reg_address + i, &value[i]) ;
        if (errStatus != I2C_MSTR_NO_ERROR) {
            break ;
        }
    }
#else 
    errStatus = I2C_MasterSendStart(i2c_slave_address, 0) ; /* write access */
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterWriteByte(reg_address) ;
    } else {
        print("I2C Send Start Failed!\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        CyDelay(i2c_delay) ;
        errStatus = I2C_MasterSendRestart(i2c_slave_address, 1) ; /* read access */
    } else {
        print("I2C Write Reg Address Failed\n\r") ;
    }
    if (errStatus == I2C_MSTR_NO_ERROR) {
        for (i = 0 ; i < num - 1 ; i++ ){
            CyDelay(i2c_delay) ;
            value[i] = I2C_MasterReadByte(I2C_ACK_DATA) ;
        }
        if (errStatus == I2C_MSTR_NO_ERROR) {
            value[i] = I2C_MasterReadByte(I2C_NAK_DATA) ;
        } else {
            print("I2C Read Reg Data Failed!\n\r") ; 
        }
    } else {
        print("I2C Send Restart Failed!\n\r") ;
    }
    errStatus = I2C_MasterSendStop() ;  
#endif
    return( errStatus ) ;    
}
#endif /* i2c__DISABLED */